package repostatus

import (
	"errors"

	"go.wit.com/log"
)

func (rs *RepoStatus) Update() {
	if !rs.Ready() {
		log.Log(WARN, "can't update yet. ready is false")
		log.Error(errors.New("Update() is not ready yet"))
		return
	}

	pb := rs.pb

	// store the current checked out branch name and version
	rs.checkCurrentBranchName()
	out := rs.pb.GetCurrentVersion()
	rs.currentVersion.SetValue(out)

	// read in the tags
	// rs.populateTags()

	// record if the repo is dirty
	pb.CheckDirty()

	// display the last tag version
	name := rs.pb.GetLastTagVersion()
	rs.lasttag.SetText(name)

	// store the master branch version
	ver := pb.GetMasterVersion()
	rs.mainBranchVersion.SetValue(ver)

	rs.develBranchVersion.SetValue(pb.GetDevelVersion())
	rs.userBranchVersion.SetValue(pb.GetUserVersion())

	// populates a string into the rs.gitState widget
	// todo: make the values from this function a bit cleaner
	rs.CheckGitState()
}

func (rs *RepoStatus) CheckGitState() string {
	state := rs.pb.GetState()
	rs.gitState.SetText(state)
	return state
}
